import { __assign, __values } from "tslib";
import { create } from 'mutative';
import { changeStateKey, identifierKey, stateKey, storeKey, bootstrappedKey, subscriptionsKey, strictKey, enablePatchesKey, } from './constant';
import { EventEmitter, getStagedState } from './utils/index';
export var createStore = function (options, preloadedState, config) {
    var e_1, _a;
    var _b, _c;
    if (config === void 0) { config = {}; }
    if (typeof options !== 'object' || !Array.isArray(options.modules)) {
        throw new Error("'createStore' options should be a object with a property 'modules'");
    }
    var strict = (_b = options.strict) !== null && _b !== void 0 ? _b : __DEV__;
    var enablePatches = (_c = config.enablePatches) !== null && _c !== void 0 ? _c : false;
    var state = {};
    var subscriptions = [];
    var identifiers = new Set();
    var eventEmitter = new EventEmitter();
    var store = {
        dispatch: function (action) {
            if (config.hook) {
                action = config.hook(store, action);
            }
            state = action._state;
            eventEmitter.emit(changeStateKey);
        },
        getState: function () { return state; },
        subscribe: function (subscription) {
            eventEmitter.on(changeStateKey, subscription);
            return function () { return eventEmitter.off(changeStateKey, subscription); };
        },
    };
    options.modules.forEach(function (module, index) {
        var _a, _b, _c;
        var _d;
        if (typeof module !== 'object' || module === null)
            return;
        var service = module;
        var className = Object.getPrototypeOf(service).constructor.name;
        if (typeof service[stateKey] === 'undefined' || service[bootstrappedKey]) {
            if (__DEV__) {
                if (service[bootstrappedKey]) {
                    console.warn("The module with an index of ".concat(index, " and a name of ").concat(className, " in the module list is a duplicate module."));
                }
            }
        }
        var identifier = (_d = service[identifierKey]) !== null && _d !== void 0 ? _d : service.name;
        if (identifier === null || typeof identifier === 'undefined') {
            identifier = "@@usm/".concat(className, "/").concat(Math.random().toString(36));
        }
        if (typeof identifier !== 'string') {
            if (__DEV__) {
                console.error("\n          Since '".concat(className, "' module has set the module state, '").concat(className, "' module must set a unique and valid class property 'name' to be used as the module index.\n          Example:\n            class FooBar {\n              name = 'FooBar'; // <- add the 'name' property.\n            }\n        "));
            }
            else {
                throw new Error("'".concat(className, "' module 'name' property should be defined as a valid 'string'."));
            }
        }
        if (identifiers.has(identifier)) {
            identifier += "".concat(index);
        }
        identifiers.add(identifier);
        var descriptors = (_a = {},
            _a[bootstrappedKey] = {
                enumerable: false,
                configurable: false,
                value: true,
            },
            _a);
        if (service[stateKey]) {
            var _loop_1 = function (key) {
                var _e, _f;
                var descriptor = Object.getOwnPropertyDescriptor(service, key);
                if (typeof descriptor === 'undefined')
                    return "continue";
                var initialValue = descriptor.value;
                if (preloadedState &&
                    preloadedState[identifier] &&
                    Object.hasOwnProperty.call(preloadedState[identifier], key)) {
                    initialValue = preloadedState[identifier][key];
                }
                Object.assign(service[stateKey], (_e = {},
                    _e[key] = initialValue,
                    _e));
                Object.assign(descriptors, (_f = {},
                    _f[key] = {
                        enumerable: true,
                        configurable: false,
                        get: function () {
                            return this[stateKey][key];
                        },
                        set: function (value) {
                            this[stateKey][key] = value;
                        },
                    },
                    _f));
            };
            for (var key in service[stateKey]) {
                _loop_1(key);
            }
            state[identifier] = strict
                ? create(__assign({}, service[stateKey]), function () { }, {
                    strict: strict,
                    enableAutoFreeze: strict,
                })
                : service[stateKey];
            Object.assign(descriptors, (_b = {},
                _b[stateKey] = {
                    enumerable: false,
                    configurable: false,
                    get: function () {
                        var _a;
                        var stagedState = getStagedState();
                        if (stagedState)
                            return stagedState[this[identifierKey]];
                        var currentState = (_a = this[storeKey]) === null || _a === void 0 ? void 0 : _a.getState()[this[identifierKey]];
                        if (strict && !Object.isFrozen(currentState)) {
                            return Object.freeze(currentState);
                        }
                        return currentState;
                    },
                },
                _b));
        }
        Object.assign(descriptors, (_c = {},
            _c[identifierKey] = {
                configurable: false,
                enumerable: false,
                value: identifier,
            },
            _c[strictKey] = {
                configurable: false,
                enumerable: false,
                value: strict,
            },
            _c[enablePatchesKey] = {
                configurable: false,
                enumerable: false,
                value: enablePatches,
            },
            _c[storeKey] = {
                configurable: false,
                enumerable: false,
                get: function () {
                    return store;
                },
            },
            _c));
        Object.defineProperties(service, descriptors);
        if (Array.isArray(service[subscriptionsKey])) {
            Array.prototype.push.apply(subscriptions, service[subscriptionsKey]);
        }
    });
    try {
        for (var subscriptions_1 = __values(subscriptions), subscriptions_1_1 = subscriptions_1.next(); !subscriptions_1_1.done; subscriptions_1_1 = subscriptions_1.next()) {
            var subscribe = subscriptions_1_1.value;
            subscribe();
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (subscriptions_1_1 && !subscriptions_1_1.done && (_a = subscriptions_1.return)) _a.call(subscriptions_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return store;
};
//# sourceMappingURL=createStore.js.map