import { storeKey, subscriptionsKey } from './constant';
import { isEqual as defaultIsEqual } from './utils/index';
var subscribe = function (module, listener) {
    var _a;
    if (typeof module !== 'object') {
        throw new Error("The subscription target '".concat(module, "' is not an object."));
    }
    var service = module;
    var className = Object.getPrototypeOf(service).constructor.name;
    if (typeof listener !== 'function') {
        throw new Error("The 'listener' should be a function in the class '".concat(className, "'."));
    }
    var unsubscribe;
    if (service[storeKey]) {
        unsubscribe = service[storeKey].subscribe(listener);
    }
    else {
        // When constructing
        var subscriptions = service[subscriptionsKey] || [];
        var _unsubscribe_1;
        subscriptions.push(function () {
            if (typeof service[storeKey] !== 'object') {
                throw new Error("The subscription target class '".concat(className, "' should be created via 'createStore()'."));
            }
            _unsubscribe_1 = service[storeKey].subscribe(listener);
        });
        unsubscribe = function () { return _unsubscribe_1(); };
        Object.assign(service, (_a = {},
            _a[subscriptionsKey] = subscriptions,
            _a));
    }
    return unsubscribe;
};
var watch = function (module, selector, watcher, _a) {
    var _b = _a === void 0 ? {} : _a, _c = _b.multiple, multiple = _c === void 0 ? false : _c, _d = _b.isEqual, isEqual = _d === void 0 ? defaultIsEqual : _d;
    if (typeof watcher !== 'function') {
        var className = Object.getPrototypeOf(module).constructor.name;
        throw new Error("The 'watcher' should be a function in the class '".concat(className, "'."));
    }
    var oldValue = selector();
    if (multiple) {
        if (!Array.isArray(oldValue)) {
            var className = Object.getPrototypeOf(module).constructor.name;
            throw new Error("The 'selector' should be a function that returns an array in the class '".concat(className, "'."));
        }
        return subscribe(module, function () {
            var newValue = selector();
            var length = oldValue.length;
            for (var i = 0; i < length; i++) {
                if (!isEqual(newValue[i], oldValue[i])) {
                    var lastValues = oldValue;
                    oldValue = newValue;
                    watcher(newValue, lastValues);
                    break;
                }
            }
        });
    }
    return subscribe(module, function () {
        var newValue = selector();
        if (!isEqual(newValue, oldValue)) {
            var lastValue = oldValue;
            oldValue = newValue;
            watcher(newValue, lastValue);
        }
    });
};
export { subscribe, watch };
//# sourceMappingURL=subscribe.js.map