import { __read, __spreadArray } from "tslib";
var EventEmitter = /** @class */ (function () {
    function EventEmitter() {
        this._events = new Map();
    }
    EventEmitter.prototype.on = function (eventType, callback, _a) {
        var _b = _a === void 0 ? {} : _a, _c = _b.once, once = _c === void 0 ? false : _c, _d = _b.priority, priority = _d === void 0 ? false : _d;
        var listeners = this._events.get(eventType) || [];
        var isExist = listeners.filter(function (listener) { return listener.callback === callback; }).length > 0;
        var listener = { callback: callback, once: once };
        if (isExist) {
            throw new Error("Event type '".concat(eventType.toString(), "' has been registered, please re-register it."));
        }
        if (priority) {
            listeners.unshift(listener);
        }
        else {
            listeners.push(listener);
        }
        this._events.set(eventType, listeners);
    };
    EventEmitter.prototype.off = function (eventType, callback) {
        var listeners = this._events.get(eventType);
        if (listeners) {
            var index = listeners.findIndex(function (listener) { return listener.callback === callback; });
            var isExist = index > -1;
            if (isExist) {
                listeners.splice(index, 1);
            }
        }
    };
    EventEmitter.prototype.remove = function (eventType) {
        if (this._events.get(eventType)) {
            return this._events.delete(eventType);
        }
        return false;
    };
    EventEmitter.prototype.emit = function (eventType) {
        var _this = this;
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        var listeners = this._events.get(eventType);
        if (!Array.isArray(listeners) || listeners.length === 0)
            return;
        __spreadArray([], __read(listeners), false).forEach(function (_a, index) {
            var callback = _a.callback, once = _a.once;
            if (once) {
                listeners.splice(index, 1);
            }
            if (Array.isArray(args))
                callback.apply(_this, args);
        });
    };
    return EventEmitter;
}());
export { EventEmitter };
//# sourceMappingURL=event.js.map